import os
import pathlib
from typing import List
from flask import send_from_directory
import marshmallow as ma
from flask_smorest.fields import Upload
from werkzeug.datastructures import FileStorage

import uuid

from ..app import Application
from ..config import APISettings
from ..logic.csv_processor import process_csv, FileTypeInvalidError
from . import formatter_routes as blp

APP = Application.get_instance()

def allowed_file(filename):
    return '.' in filename and filename.rsplit('.', 1)[1].lower() in APISettings.ALLOWED_EXTENSIONS

class MultipartFormSchema(ma.Schema):
    username = ma.fields.String(required=True)
    friendcode = ma.fields.String(required=False)

class MultipartFileSchema(ma.Schema):
    servantdata = Upload()
    cedata = Upload()

@blp.route("/", methods=["POST"])
@blp.arguments(MultipartFormSchema, location="form")
@blp.arguments(MultipartFileSchema, location="files") 
@blp.response(200)
def upload_file(form: dict[str, str], files: dict[str, FileStorage]):
    filepaths: List[pathlib.Path] = []
    returndata = {
        "username": form["username"],
        "friendcode": form.get("friendcode", "")
    }

    for name, file in files.items():
        if name not in ("servantdata", "cedata"):
            return send_from_directory(APISettings.PAGES_DIRECTORY, "error.html")

        filepath = APISettings.FILE_SAVE_DIRECTORY / f"{uuid.uuid4()}.csv"
        file.save(filepath)

        if os.stat(filepath).st_size < 1 or not allowed_file(file.filename):
            filepath.unlink()
            continue

        filepaths.append(filepath)

    if len(filepaths) == 0:
        return send_from_directory(APISettings.PAGES_DIRECTORY, "error.html")

    try:
        for f in filepaths:
            result = process_csv(f)
            returndata = returndata | result
            f.unlink()
    except FileTypeInvalidError:
        for f in filepaths:
            f.unlink()
        return send_from_directory(APISettings.PAGES_DIRECTORY, "error.html")

    return returndata

@blp.route("/", methods=["GET"])
def file_dialog():
    return send_from_directory(APISettings.PAGES_DIRECTORY, "index.html")
